@@ -289,6 +289,9 @@ class NEUSetter:
                     'PalletCount',
                     'ShelfCount',
                     'CaseCount',
+                    'Expiry',
+                    'BestBy',
+                    'AquisitionDate',
                     ]
                 fields.extend(LOCATION_FIELDS)
                 fct=len(fields)
