@@ -27,7 +27,7 @@ import itertools
 import decimal
 from decimal import localcontext,Decimal
 unit_registry=pint.UnitRegistry()
-import math
+import math,scipy
 from radboy.HowDoYouDefineMe.CoreEmotions import *
 
 def volume():
@@ -1786,4 +1786,310 @@ def USCurrencyMassValues():
             
         except Exception as e:
             print(e)
-            continue
\ No newline at end of file
+            continue
+
+
+def golden_ratio():
+    msg=''
+    while True:
+        try:
+            fields={
+                'measurement':{
+                'type':'dec.dec',
+                'default':48,
+                },
+            }
+            fd=FormBuilder(data=fields,passThruText=msg)
+            if fd is None:
+                return
+            side1_value=(decc(fd['measurement'])/decc(scipy.constants.golden_ratio))
+            side2_value=fd['measurement']-decc(side1_value)
+            which=Control(func=FormBuilderMkText,ptext=f"Which side do you wish to return [for a side of {fd['measurement']}: side1_value={side1_value},side2_value={side2_value}]?",helpText="yes/1/true=side 1,side 2 is false/no/0",data="boolean")
+            if which in [None,"NaN"]:
+                return
+            elif which:
+                return side1_value
+            else:
+                return side2_value
+        except Exception as e:
+            print(e)
+
+
+def currency_conversion():
+    cvt_registry=pint.UnitRegistry()
+    
+    definition=f'''
+    USD = [currency]
+Argentine_Peso  =    nan usd
+Australian_Dollar   =    nan usd
+Bahraini_Dinar  =    nan usd
+Botswana_Pula   =    nan usd
+Brazilian_Real  =    nan usd
+British_Pound   =    nan usd
+Bruneian_Dollar =    nan usd
+Bulgarian_Lev   =    nan usd
+Canadian_Dollar =    nan usd
+Chilean_Peso    =    nan usd
+Chinese_Yuan_Renminbi   =    nan usd
+Colombian_Peso  =    nan usd
+Czech_Koruna    =    nan usd
+Danish_Krone    =    nan usd
+Emirati_Dirham  =    nan usd
+Euro    =    nan usd
+Hong_Kong_Dollar    =    nan usd
+Hungarian_Forint    =    nan usd
+Icelandic_Krona =    nan usd
+Indian_Rupee    =    nan usd
+Indonesian_Rupiah   =    nan usd
+Iranian_Rial    =    nan usd
+Israeli_Shekel  =    nan usd
+Japanese_Yen    =    nan usd
+Kazakhstani_Tenge   =    nan usd
+Kuwaiti_Dinar   =    nan usd
+Libyan_Dinar    =    nan usd
+Malaysian_Ringgit   =    nan usd
+Mauritian_Rupee =    nan usd
+Mexican_Peso    =    nan usd
+Nepalese_Rupee  =    nan usd
+New_Zealand_Dollar  =    nan usd
+Norwegian_Krone =    nan usd
+Omani_Rial  =    nan usd
+Pakistani_Rupee =    nan usd
+Philippine_Peso =    nan usd
+Polish_Zloty    =    nan usd
+Qatari_Riyal    =    nan usd
+Romanian_New_Leu    =    nan usd
+Russian_Ruble   =    nan usd
+Saudi_Arabian_Riyal =    nan usd
+Singapore_Dollar    =    nan usd
+South_African_Rand  =    nan usd
+South_Korean_Won    =    nan usd
+Sri_Lankan_Rupee    =    nan usd
+Swedish_Krona   =    nan usd
+Swiss_Franc =    nan usd
+Taiwan_New_Dollar   =    nan usd
+Thai_Baht   =    nan usd
+Trinidadian_Dollar  =    nan usd
+Turkish_Lira    =    nan usd
+
+@context FX
+Argentine_Peso  =    0.000671    usd
+Australian_Dollar   =    0.651104    usd
+Bahraini_Dinar  =    2.659574    usd
+Botswana_Pula   =    0.070042    usd
+Brazilian_Real  =    0.185537    usd
+British_Pound   =    1.330948    usd
+Bruneian_Dollar =    0.769854    usd
+Bulgarian_Lev   =    0.594475    usd
+Canadian_Dollar =    0.714527    usd
+Chilean_Peso    =    0.001062    usd
+Chinese_Yuan_Renminbi   =    0.140424    usd
+Colombian_Peso  =    0.000259    usd
+Czech_Koruna    =    0.047793    usd
+Danish_Krone    =    0.155642    usd
+Emirati_Dirham  =    0.272294    usd
+Euro    =    1.162692    usd
+Hong_Kong_Dollar    =    0.128701    usd
+Hungarian_Forint    =    0.002981    usd
+Icelandic_Krona =    0.008119    usd
+Indian_Rupee    =    0.011384    usd
+Indonesian_Rupiah   =    0.00006 usd
+Iranian_Rial    =    0.000024    usd
+Israeli_Shekel  =    0.304734    usd
+Japanese_Yen    =    0.006545    usd
+Kazakhstani_Tenge   =    0.00186 usd
+Kuwaiti_Dinar   =    3.261214    usd
+Libyan_Dinar    =    0.183824    usd
+Malaysian_Ringgit   =    0.236753    usd
+Mauritian_Rupee =    0.02197 usd
+Mexican_Peso    =    0.054181    usd
+Nepalese_Rupee  =    0.007112    usd
+New_Zealand_Dollar  =    0.575051    usd
+Norwegian_Krone =    0.099905    usd
+Omani_Rial  =    2.603489    usd
+Pakistani_Rupee =    0.003531    usd
+Philippine_Peso =    0.017016    usd
+Polish_Zloty    =    0.274017    usd
+Qatari_Riyal    =    0.274725    usd
+Romanian_New_Leu    =    0.228593    usd
+Russian_Ruble   =    0.012559    usd
+Saudi_Arabian_Riyal =    0.266667    usd
+Singapore_Dollar    =    0.769854    usd
+South_African_Rand  =    0.057932    usd
+South_Korean_Won    =    0.000695    usd
+Sri_Lankan_Rupee    =    0.003293    usd
+Swedish_Krona   =    0.106347    usd
+Swiss_Franc =    1.256685    usd
+Taiwan_New_Dollar   =    0.032417    usd
+Thai_Baht   =    0.030604    usd
+Trinidadian_Dollar  =    0.147095    usd
+Turkish_Lira    =    0.023829    usd
+@end'''.lower()
+    defFile=db.detectGetOrSet("currency_definitions_file","currency_definitions.txt",setValue=False,literal=True)
+    if defFile is None:
+        return
+    defFile=Path(defFile)
+    with open(defFile,"w") as out:
+        out.write(definition)
+    cvt_registry.load_definitions(defFile)
+    with cvt_registry.context("fx") as cvtr:  
+        while True:  
+            try:
+                htext=[]
+                definition=definition.split("@context FX")[-1].replace('\n@end','')
+                cta=len(definition.split("\n"))
+                formats='\n'.join([std_colorize(i,num,cta) for num,i in enumerate(definition.split("\n"))])
+                
+                formats=f'''Conversion Formats are:\n{formats}\n'''
+                fields={
+                'value':{
+                    'default':1,
+                    'type':'float',
+                },
+                'FromString':{
+                    'default':'USD',
+                    'type':'string',
+                },
+                'ToString':{
+                    'default':'Euro',
+                    'type':'string'
+                },
+                }
+                fb=FormBuilder(data=fields,passThruText=formats)
+                if fb is None:
+                    return
+
+                return_string=fb['ToString'].lower()
+                value_string=f"{fb['value']} {fb['FromString']}".lower()
+                resultant=cvtr.Quantity(value_string).to(return_string)
+
+                #if it gets here return None
+                return resultant
+            except Exception as e:
+                print(e)
+
+def bible_try():
+    try:
+        os.system("sonofman")
+        return None
+    except Exception as e:
+        print(e)
+
+DELCHAR=db.detectGetOrSet("DELCHAR preloader func","|",setValue=False,literal=True)
+if not DELCHAR:
+    DELCHAR='|'
+
+def SalesFloorLocationString():
+    fields=OrderedDict({
+
+        'Aisle[s]':{
+            'default':'',
+            'type':'string',
+        },
+        'Bay[s]/AisleDepth':{
+            'default':'',
+            'type':'string',
+        },
+        'Shel[f,ves]':{
+            'default':'',
+            'type':'string',
+        }
+    })
+    passThruText=f"""
+{Fore.orange_red_1}Valid Aisle[s]:{Fore.grey_85}
+    this is the aisle on which the product resides
+    if the product belongs on an end cap use the endcap here as
+    well. endcaps are numbered 0+=1 from left to right of the store
+    front. the same is true for aisle. if the product resides on an 
+    end cap, append FEC (0FEC) to signify front end cap 0, or 0REC for 
+    rear end cap 0.
+    0 -> on aisle 0
+    0,1 -> on aisle 0 and  1
+    0-2 -> from aisle 0 to aisle 2
+
+    encaps on the front side of the aisle will have an FEC Appended to its number
+     The same is true for rear of the aisle(REC). if a encap range is used, the character
+     will identify its side of the store. if two encaps on opposite sides of the store
+     are specified, then use 2 separate ranges; one for front and one for the rear.
+    0FEC -> on front endcap 0
+    0FEC,1REC -> on front endcap 0 and on rear endcap 1.
+    0-2FEC -> from front endcap 0 to front endcap 2
+    0-2REC -> from rear endcap 0 to rear endcap 2
+    0-2FEC,0-2REC -> from front endcap 0 to front endcap 2 && from rear endcap 0 to rear endcap 2
+
+{Fore.orange_red_1}Valid 'Bay[s]/AisleDepth':{Fore.grey_85}
+    This is How many shelf bays deep from the front of the store
+     to the back, where 0 is the first bay from the endcap at the 
+     front of the store and increments upwards to the rear end cap.
+     Bays on the right side of the aisle will have an R Appended to its number
+     The same is true for left of the aisle. if a bay range is used, the character
+     will identify its side of the aisle. if two bays on opposite sides of the aisle
+     are specified, then use 2 separate ranges; one for left and one for right.
+    0 -> on bay 0
+    0,1 -> on bay 0 and  1
+    0-2R -> from bay 0 to bay 2 on the right side of the aisle
+    0-2L -> from bay 0 to bay 2 on the left side of the aisle
+    0-2R,0-2L -> from bay 0 to bay 2 on the right side of the aisle && from bay 0 to bay 2 on the left side of the aisle.
+{Fore.orange_red_1}Valid Shel[f,ves]:{Fore.grey_85}
+    this is the height where the product is on the shelf
+    shelves are number 0 to their highest from bottom to top
+    where the very bottom shelf is 0, and the next following shelf upwards is 
+    1, and so on.
+    0 -> on shelf 0
+    0,1 -> on shelf 0 and  1
+    0-2 -> from shelf 0 to shelf 2
+{Fore.light_green}Aisle or EndCap{Fore.light_red}/{Fore.light_yellow}Depth or Bay in Aisle{Fore.light_red}/{Fore.light_steel_blue}Shelf or Location Number(s) Where Item Resides [optional]{Style.reset}"""
+    fb=FormBuilder(data=fields,passThruText=passThruText)
+    if fb is None:
+        return
+    if fb['Shel[f,ves]'] not in ['',]:
+        fb['Shel[f,ves]']=f"{DELCHAR}{fb['Shel[f,ves]']}"
+    locationString=f"{fb['Aisle[s]']}{DELCHAR}{fb['Bay[s]/AisleDepth']}{fb['Shel[f,ves]']}"
+    return locationString
+
+def BackroomLocation():
+    fields=OrderedDict({
+        'moduleType':{
+            'default':'',
+            'type':'string',
+            },
+        'moduleNumberRange':{
+            'default':'',
+            'type':'string',
+        },
+        'caseID':{
+            'default':'',
+            'type':'string',
+        },
+
+    })
+    passThruText=f'''
+{Fore.orange_red_1}Valid ModuleTypes:{Fore.grey_85}
+    this what to look on for where the product is stored
+    s or S -> For Pallet/Platform [General]/Skid
+    u or U -> For U-Boat/Size Wheeler
+    rc or rC or Rc or RC ->  for RotaCart
+    sc or sC or Sc or SC -> Shopping Cart
+
+{Fore.orange_red_1}Valid ModuleNumberRange:{Fore.grey_85}
+    This which of the what's contains said item.
+    0 -> on which 0
+    0,1 -> on which 0 and which 1
+    0-2 -> from which 0 to which 2 on the left side of the aisle
+{Fore.orange_red_1}Valid caseID:{Fore.grey_85}
+    This is the case where the item is stored in which is the which found on the what.
+    anything that you want as long as it is unique to the module in which it is stored.
+     try using the following cmds for something on the fly.
+     nanoid - nanoid
+     crbc - checked random barcode
+     dsur - generate a datestring
+     urid - generate reciept id with a log
+     cruid - checked uuid
+    '''
+    fb=FormBuilder(data=fields,passThruText=passThruText)
+    if fb is None:
+        return
+    if fb['caseID'] not in ['',]:
+        fb['caseID']=f"{DELCHAR}{fb['caseID']}"
+    locationString=f"{fb['moduleType']}{DELCHAR}{fb['moduleNumberRange']}{' '.join(db.stre(fb['caseID'])/3)}"
+    return locationString
